/*
 *  Copyright (C) 2002-2003  Mattia Dongili<dongili@supereva.it>
 *                           George Staikos <staikos@0wned.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <limits.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <time.h>
#include <string.h>
#include <syslog.h>
#include <signal.h>
#include <ctype.h>
#include <dirent.h>
#include <dlfcn.h>

#include "string_list.h"

#define _CPUFREQD_VERSION_ 	"1.0-beta1"

#define CPUFREQD_CONFIG   "/etc/cpufreqd.conf"
#define CPUFREQD_PIDFILE  "/var/run/cpufreqd.pid"
#define APM_PROC_FILE     "/proc/apm"
#define ACPI_PROC_DIR     "/proc/acpi/"
#define ACPI_BATTERY_DIR  "/proc/acpi/battery/"
#define ACPI_AC_DIR       "/proc/acpi/ac_adapter/"

#define CPUFREQ_SYSFS_INTERFACE 	      "/sys/devices/sys/cpu0"
#define CPUFREQ_SYSFS_INTERFACE_POLICY  "/sys/devices/sys/cpu0/scaling_governor"
#define CPUFREQ_SYSFS_INTERFACE_MAX     "/sys/devices/sys/cpu0/scaling_max_freq"
#define CPUFREQ_SYSFS_INTERFACE_MIN     "/sys/devices/sys/cpu0/scaling_min_freq"
#define CPUFREQ_PROC_INTERFACE          "/proc/cpufreq"
#define CPUFREQ_PROC_INTERFACE_SPEED    "/proc/sys/cpu/0/speed"
#define CPUFREQ_PROC_INTERFACE_MIN      "/proc/sys/cpu/0/speed-min"
#define CPUFREQ_PROC_INTERFACE_MAX      "/proc/sys/cpu/0/speed-max"

#define PM_TYPE_APM   0
#define PM_TYPE_ACPI  1

typedef struct {
  short int min;
  short int max;
} battery_interval;

typedef struct profile {
  char name[255];
  short int min_freq;
  short int max_freq;
  char policy_name[255];
  struct profile *next;
} profile;

/* TODO: add precedence factor for rules having he same score */
/* TODO: add a command to be executed after the rule is activated */
typedef struct rule {
  char name[255];
  int ac;        /* AC power state */
  battery_interval *bat;
  char profile_name[255];
  struct string_list *program_list;
  profile *prof;
  struct rule *next;
} rule;

typedef struct sys_info {
  int has_battery:1;
  int ac;
  int battery_percent;
  int battery_time;
  short int temp;
  int flags;
	char version[101];
	char time[101];
} sys_info;

#define PM_TYPE_APM   0
#define PM_TYPE_ACPI  1

#define MATCH_AC       4
#define MATCH_BATTERY  2
#define MATCH_PROG     1

typedef struct {
  char config_file[512];
  char pidfile[512];
  short int poll_interval;
  short int pm_type;
  short int has_sysfs;
  short int no_daemon;
  rule *rules;
  profile *profiles;
} general;

#include "main.h"
#include "config_parser.h"
