/*
 *  Copyright (C) 2002-2003  Mattia Dongili<dongili@supereva.it>
 *                           George Staikos <staikos@0wned.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "cpufreqd.h"

char (*bat_path)[255] = 0L;
char ac_file[255];
char version[101];
short int err = 0;
int bat_num = 0;

/* int no_dots(const struct dirent *d)
 * 
 * Filter function for scandir, returns
 * 0 if the first char of d_name is not
 * a dot.
 */
int no_dots(const struct dirent *d) {
  return d->d_name[0]!= '.';
}

/* void libsys_init(void)
 *
 * Initialize local variables.
 */
void libsys_init(void) {
  
  DIR *dp;
  FILE *fp;
  struct dirent **namelist;
  struct dirent *direntry;
  int n = 0;
  
  fp = fopen(ACPI_PROC_DIR"info", "r");
  if (!fp) {
    syslog(LOG_ERR, "libsys_init(): %s: %s\n", ACPI_PROC_DIR, strerror(errno));
    err++;
    return;
  }
  fscanf(fp, "version: %s", version);
  fclose(fp);

  /* get batteries paths */
  bat_num = n = scandir(ACPI_BATTERY_DIR, &namelist, no_dots, alphasort);
  if (n < 0)
    printf("libsys_init() - scandir: %s\n", strerror(errno));
  else {
    bat_path = (char (*)[])malloc(bat_num*255*sizeof(char));

    while(n--) {
      snprintf(bat_path[n], 255, "%s%s", ACPI_BATTERY_DIR, namelist[n]->d_name);
      free(namelist[n]);
    }
  }
  free(namelist);
        
  /* get AC path */
  dp = opendir(ACPI_AC_DIR);
  if (!dp) {
    syslog(LOG_ERR, "scan_system_info(): %s: %s\n", ACPI_AC_DIR, strerror(errno));
    err++;
    return;
  }

  while((direntry=readdir(dp)) != NULL) {
    if (direntry->d_name[0] == '.') /* skip dot entries */
      continue;

    snprintf(ac_file, 255, "%s%s/state", ACPI_AC_DIR, direntry->d_name);
    break; /* as soon as we fill ac_file there's no need to go on */
  }                              
  closedir(dp);
}

/* void libsys_fini (void)
 *
 * Free allocated structures.
 */
void libsys_fini (void) {
  if (bat_path)
    free(bat_path);
}

/* int scan_system_info(sys_info *s) 
 *
 * Reads ACPI info and fills the input sys_info struct.
 * Implements the prototype needed by cpufreqd to get
 * info from the PM layer.
 * 
 * Returns 0 on success, -1 otherwise
 *
 */
int scan_system_info(sys_info *s) {

  FILE *fp;
  char buf[101];

  int i = 0;
  float bat_full_capacity = 0;
  float bat_total_capacity = 0;
  char line[5][101];
  char file_name[256];
  
  /* check if _init was successful */
  if (err>0)
    return -1;
  
  /* set version information */
  snprintf(s->version, 255, "%s", version);
  
  /* Read battery informations */
  for (i=0; i<bat_num; i++) {
    /** /proc/acpi/battery/.../info **/
    snprintf(file_name, 255, "%s/info", bat_path[i]);
    fp = fopen(file_name, "r");
    if (!fp) {
      syslog(LOG_ERR, "scan_system_info(): %s: %s\n", file_name, strerror(errno));
      return(-1);
    }

    while (fgets(buf, 100, fp)) {
      memset(line, 0, 5*101);
      sscanf(buf, "%s %s %s %s %s\n", line[0], line[1], line[2], line[3], line[4]);
      /* present: yes */
      if (strcmp(line[0], "present:")==0 && s->has_battery==0) {
        if (strcmp(line[1], "yes")==0) {
          s->has_battery = 1;
        } else {
          s->has_battery = 0;
          break; /* no need to go on */
        }
      }

      /* last full capacity: 37740 mWh */
      if (strcmp(line[0], "last")==0 &&
          strcmp(line[1], "full")==0 &&
          strcmp(line[2], "capacity:")==0) {
        bat_full_capacity += atof(line[3]);
      }

    }
    fclose(fp);
    
    /* check battery state if there is at least 1 battery */
    if (s->has_battery) {
      /** /proc/acpi/battery/.../state **/
      snprintf(file_name, 255, "%s/state", bat_path[i]);
      fp = fopen(file_name, "r");
      if (!fp) {
        syslog(LOG_ERR, "scan_system_info(): %s: %s\n", file_name, strerror(errno));
        return(-1);
      }

      /* calculates medium battery life between all batteries */
      while (fgets(buf, 100, fp)) {
        memset(line, 0, 5*100);
        sscanf(buf, "%s %s %s %s %s\n", line[0], line[1], line[2], line[3], line[4]);
        /* remaining capacity: 34010 mWh */
        if (strcmp(line[0], "remaining")==0) {
          bat_total_capacity += atof(line[2]);
          break;
        }
      }
      fclose(fp);
    } /* end if has_battery*/
    
  } /* end bat_path loop */

  s->battery_percent = 100 * (bat_total_capacity / (double)bat_full_capacity) / bat_num;

  /* Read AC adapter state */
  fp = fopen(ac_file, "r");
  if (!fp) {
    syslog(LOG_ERR, "scan_system_info(): %s: %s\n", ac_file, strerror(errno));
    return(-1);
  }

  while (fgets(buf, 100, fp)) {
    memset(line, 0, 2*101);
    sscanf(buf, "%s %s\n", line[0], line[1]);
    /* state: on-line */
    if (strcmp(line[0], "state:")==0) {
      s->ac = strcmp(line[1], "on-line")==0 ? 1 : 0;
      break;
    }
  }
  fclose(fp);

  /*
  syslog(LOG_INFO, "scan_system_info(): battery %s - %d - %s\n",
                    s->has_battery?"present":"absent", 
                    s->battery_percent, 
                    s->ac?"on-line":"off-line");
  */
  return 0;
}

